@@ -1,7 +1,7 @@
 /***************************************************************************
 **                                                                        **
 **  QCustomPlot, an easy to use, modern plotting widget for Qt            **
-**  Copyright (C) 2011-2017 Emanuel Eichhammer                            **
+**  Copyright (C) 2011-2021 Emanuel Eichhammer                            **
 **                                                                        **
 **  This program is free software: you can redistribute it and/or modify  **
 **  it under the terms of the GNU General Public License as published by  **
@@ -19,8 +19,8 @@
 ****************************************************************************
 **           Author: Emanuel Eichhammer                                   **
 **  Website/Contact: http://www.qcustomplot.com/                          **
-**             Date: 04.09.17                                             **
-**          Version: 2.0.0                                                **
+**             Date: 29.03.21                                             **
+**          Version: 2.1.0                                                **
 ****************************************************************************/
 
 #include "layer.h"
@@ -134,10 +134,10 @@ QCPLayer::~QCPLayer()
   // call QCustomPlot::removeLayer, which moves all layerables off this layer before deleting it.)
   
   while (!mChildren.isEmpty())
-    mChildren.last()->setLayer(0); // removes itself from mChildren via removeChild()
+    mChildren.last()->setLayer(nullptr); // removes itself from mChildren via removeChild()
   
   if (mParentPlot->currentLayer() == this)
-    qDebug() << Q_FUNC_INFO << "The parent plot's mCurrentLayer will be a dangling pointer. Should have been set to a valid layer or 0 beforehand.";
+    qDebug() << Q_FUNC_INFO << "The parent plot's mCurrentLayer will be a dangling pointer. Should have been set to a valid layer or nullptr beforehand.";
 }
 
 /*!
@@ -179,8 +179,8 @@ void QCPLayer::setMode(QCPLayer::LayerMode mode)
   if (mMode != mode)
   {
     mMode = mode;
-    if (!mPaintBuffer.isNull())
-      mPaintBuffer.data()->setInvalidated();
+    if (QSharedPointer<QCPAbstractPaintBuffer> pb = mPaintBuffer.toStrongRef())
+      pb->setInvalidated();
   }
 }
 
@@ -215,18 +215,18 @@ void QCPLayer::draw(QCPPainter *painter)
 */
 void QCPLayer::drawToPaintBuffer()
 {
-  if (!mPaintBuffer.isNull())
+  if (QSharedPointer<QCPAbstractPaintBuffer> pb = mPaintBuffer.toStrongRef())
   {
-    if (QCPPainter *painter = mPaintBuffer.data()->startPainting())
+    if (QCPPainter *painter = pb->startPainting())
     {
       if (painter->isActive())
         draw(painter);
       else
         qDebug() << Q_FUNC_INFO << "paint buffer returned inactive painter";
       delete painter;
-      mPaintBuffer.data()->donePainting();
+      pb->donePainting();
     } else
-      qDebug() << Q_FUNC_INFO << "paint buffer returned zero painter";
+      qDebug() << Q_FUNC_INFO << "paint buffer returned nullptr painter";
   } else
     qDebug() << Q_FUNC_INFO << "no valid paint buffer associated with this layer";
 }
@@ -236,27 +236,28 @@ void QCPLayer::drawToPaintBuffer()
   the layerables on this specific layer, without the need to replot all other layers (as a call to
   \ref QCustomPlot::replot would do).
 
+  QCustomPlot also makes sure to replot all layers instead of only this one, if the layer ordering
+  or any layerable-layer-association has changed since the last full replot and any other paint
+  buffers were thus invalidated.
+
   If the layer mode is \ref lmLogical however, this method simply calls \ref QCustomPlot::replot on
   the parent QCustomPlot instance.
 
-  QCustomPlot also makes sure to replot all layers instead of only this one, if the layer ordering
-  has changed since the last full replot and the other paint buffers were thus invalidated.
-
   \see draw
 */
 void QCPLayer::replot()
 {
   if (mMode == lmBuffered && !mParentPlot->hasInvalidatedPaintBuffers())
   {
-    if (!mPaintBuffer.isNull())
+    if (QSharedPointer<QCPAbstractPaintBuffer> pb = mPaintBuffer.toStrongRef())
     {
-      mPaintBuffer.data()->clear(Qt::transparent);
+      pb->clear(Qt::transparent);
       drawToPaintBuffer();
-      mPaintBuffer.data()->setInvalidated(false);
+      pb->setInvalidated(false); // since layer is lmBuffered, we know only this layer is on buffer and we can reset invalidated flag
       mParentPlot->update();
     } else
       qDebug() << Q_FUNC_INFO << "no valid paint buffer associated with this layer";
-  } else if (mMode == lmLogical)
+  } else
     mParentPlot->replot();
 }
 
@@ -278,8 +279,8 @@ void QCPLayer::addChild(QCPLayerable *layerable, bool prepend)
       mChildren.prepend(layerable);
     else
       mChildren.append(layerable);
-    if (!mPaintBuffer.isNull())
-      mPaintBuffer.data()->setInvalidated();
+    if (QSharedPointer<QCPAbstractPaintBuffer> pb = mPaintBuffer.toStrongRef())
+      pb->setInvalidated();
   } else
     qDebug() << Q_FUNC_INFO << "layerable is already child of this layer" << reinterpret_cast<quintptr>(layerable);
 }
@@ -297,8 +298,8 @@ void QCPLayer::removeChild(QCPLayerable *layerable)
 {
   if (mChildren.removeOne(layerable))
   {
-    if (!mPaintBuffer.isNull())
-      mPaintBuffer.data()->setInvalidated();
+    if (QSharedPointer<QCPAbstractPaintBuffer> pb = mPaintBuffer.toStrongRef())
+      pb->setInvalidated();
   } else
     qDebug() << Q_FUNC_INFO << "layerable is not child of this layer" << reinterpret_cast<quintptr>(layerable);
 }
@@ -329,7 +330,8 @@ void QCPLayer::removeChild(QCPLayerable *layerable)
   only get drawn if their parent layerables are visible, too.
   
   Note that a parent layerable is not necessarily also the QObject parent for memory management.
-  Further, a layerable doesn't always have a parent layerable, so this function may return 0.
+  Further, a layerable doesn't always have a parent layerable, so this function may return \c
+  nullptr.
   
   A parent layerable is set implicitly when placed inside layout elements and doesn't need to be
   set manually by the user.
@@ -401,8 +403,8 @@ void QCPLayer::removeChild(QCPLayerable *layerable)
   targetLayer is an empty string, it places itself on the current layer of the plot (see \ref
   QCustomPlot::setCurrentLayer).
   
-  It is possible to provide 0 as \a plot. In that case, you should assign a parent plot at a later
-  time with \ref initializeParentPlot.
+  It is possible to provide \c nullptr as \a plot. In that case, you should assign a parent plot at
+  a later time with \ref initializeParentPlot.
   
   The layerable's parent layerable is set to \a parentLayerable, if provided. Direct layerable
   parents are mainly used to control visibility in a hierarchy of layerables. This means a
@@ -416,7 +418,7 @@ QCPLayerable::QCPLayerable(QCustomPlot *plot, QString targetLayer, QCPLayerable
   mVisible(true),
   mParentPlot(plot),
   mParentLayerable(parentLayerable),
-  mLayer(0),
+  mLayer(nullptr),
   mAntialiased(true)
 {
   if (mParentPlot)
@@ -433,7 +435,7 @@ QCPLayerable::~QCPLayerable()
   if (mLayer)
   {
     mLayer->removeChild(this);
-    mLayer = 0;
+    mLayer = nullptr;
   }
 }
 
@@ -539,9 +541,14 @@ bool QCPLayerable::realVisibility() const
   placed in \a details. So in the subsequent \ref selectEvent, the decision which part was
   selected doesn't have to be done a second time for a single selection operation.
   
-  You may pass 0 as \a details to indicate that you are not interested in those selection details.
+  In the case of 1D Plottables (\ref QCPAbstractPlottable1D, like \ref QCPGraph or \ref QCPBars) \a
+  details will be set to a \ref QCPDataSelection, describing the closest data point to \a pos.
   
-  \see selectEvent, deselectEvent, mousePressEvent, wheelEvent, QCustomPlot::setInteractions
+  You may pass \c nullptr as \a details to indicate that you are not interested in those selection
+  details.
+  
+  \see selectEvent, deselectEvent, mousePressEvent, wheelEvent, QCustomPlot::setInteractions,
+  QCPAbstractPlottable1D::selectTestRect
 */
 double QCPLayerable::selectTest(const QPointF &pos, bool onlySelectable, QVariant *details) const
 {
@@ -554,11 +561,11 @@ double QCPLayerable::selectTest(const QPointF &pos, bool onlySelectable, QVarian
 /*! \internal
   
   Sets the parent plot of this layerable. Use this function once to set the parent plot if you have
-  passed 0 in the constructor. It can not be used to move a layerable from one QCustomPlot to
-  another one.
+  passed \c nullptr in the constructor. It can not be used to move a layerable from one QCustomPlot
+  to another one.
   
-  Note that, unlike when passing a non-null parent plot in the constructor, this function does not
-  make \a parentPlot the QObject-parent of this layerable. If you want this, call
+  Note that, unlike when passing a non \c nullptr parent plot in the constructor, this function
+  does not make \a parentPlot the QObject-parent of this layerable. If you want this, call
   QObject::setParent(\a parentPlot) in addition to this function.
   
   Further, you will probably want to set a layer (\ref setLayer) after calling this function, to
@@ -651,8 +658,8 @@ void QCPLayerable::applyAntialiasingHint(QCPPainter *painter, bool localAntialia
 /*! \internal
 
   This function is called by \ref initializeParentPlot, to allow subclasses to react on the setting
-  of a parent plot. This is the case when 0 was passed as parent plot in the constructor, and the
-  parent plot is set at a later time.
+  of a parent plot. This is the case when \c nullptr was passed as parent plot in the constructor,
+  and the parent plot is set at a later time.
   
   For example, QCPLayoutElement/QCPLayout hierarchies may be created independently of any
   QCustomPlot at first. When they are then added to a layout inside the QCustomPlot, the top level
@@ -699,7 +706,7 @@ QRect QCPLayerable::clipRect() const
   if (mParentPlot)
     return mParentPlot->viewport();
   else
-    return QRect();
+    return {};
 }
 
 /*! \internal
@@ -792,7 +799,7 @@ void QCPLayerable::mousePressEvent(QMouseEvent *event, const QVariant &details)
 
   The current pixel position of the cursor on the QCustomPlot widget is accessible via \c
   event->pos(). The parameter \a startPos indicates the position where the initial \ref
-  mousePressEvent occured, that started the mouse interaction.
+  mousePressEvent occurred, that started the mouse interaction.
 
   The default implementation does nothing.
 
@@ -810,7 +817,7 @@ void QCPLayerable::mouseMoveEvent(QMouseEvent *event, const QPointF &startPos)
 
   The current pixel position of the cursor on the QCustomPlot widget is accessible via \c
   event->pos(). The parameter \a startPos indicates the position where the initial \ref
-  mousePressEvent occured, that started the mouse interaction.
+  mousePressEvent occurred, that started the mouse interaction.
 
   The default implementation does nothing.
 
@@ -862,10 +869,10 @@ void QCPLayerable::mouseDoubleClickEvent(QMouseEvent *event, const QVariant &det
   The current pixel position of the cursor on the QCustomPlot widget is accessible via \c
   event->pos().
 
-  The \c event->delta() indicates how far the mouse wheel was turned, which is usually +/- 120 for
-  single rotation steps. However, if the mouse wheel is turned rapidly, multiple steps may
-  accumulate to one event, making \c event->delta() larger. On the other hand, if the wheel has
-  very smooth steps or none at all, the delta may be smaller.
+  The \c event->angleDelta() indicates how far the mouse wheel was turned, which is usually +/- 120
+  for single rotation steps. However, if the mouse wheel is turned rapidly, multiple steps may
+  accumulate to one event, making the delta larger. On the other hand, if the wheel has very smooth
+  steps or none at all, the delta may be smaller.
 
   The default implementation does nothing.
 
