@@ -7,13 +7,17 @@ MainWindow::MainWindow(QWidget *parent) :
 {
   ui->setupUi(this);
   setGeometry(300, 300, 500, 500);
+  mStatusBarLabel = new QLabel(this);
+  ui->statusBar->addPermanentWidget(mStatusBarLabel);
+  
   mCustomPlot = new QCustomPlot(this);
   QHBoxLayout *layout = new QHBoxLayout();
   ui->centralWidget->setLayout(layout);
   layout->insertWidget(0, mCustomPlot);
   mCustomPlot->axisRect()->setupFullAxesBox(true);
   connect(mCustomPlot, SIGNAL(mousePress(QMouseEvent*)), this, SLOT(testbedMouseClick(QMouseEvent*)));
-
+  connect(mCustomPlot, SIGNAL(afterReplot()), this, SLOT(showReplotTime()));
+  
   presetInteractive(mCustomPlot);
   //setupItemAnchorTest(mCustomPlot);
   //setupItemTracerTest(mCustomPlot);
@@ -43,7 +47,9 @@ MainWindow::MainWindow(QWidget *parent) :
   //setupDataSelectTest(mCustomPlot);
   //setupErrorBarTest(mCustomPlot);
   //setupScatterSkipTest(mCustomPlot);
-  setupTestbed(mCustomPlot);
+  //setupTimeZoneTest(mCustomPlot);
+  setupPolarAxisTest(mCustomPlot);
+  //setupTestbed(mCustomPlot);
 }
 
 MainWindow::~MainWindow()
@@ -146,7 +152,7 @@ void MainWindow::setupItemTracerTest(QCustomPlot *customPlot)
   QVector<double> x(n), y(n);
   for (int i=0; i<n; ++i)
   {
-    x[i] = 0.5+i/(double)n*4;
+    x[i] = 0.5+i/double(n)*4;
     y[i] = qSin(x[i])+1.5;
   }
   graph->setData(x, y);
@@ -192,7 +198,7 @@ void MainWindow::setupGraphTest(QCustomPlot *customPlot)
 
   QSharedPointer<QCPGraphDataContainer> data(new QCPGraphDataContainer);
   int n = 10e6;
-  QTime t;
+  QElapsedTimer t;
   t.start();
   for (int i=0; i<n; ++i)
   {
@@ -244,7 +250,7 @@ void MainWindow::setupExportTest(QCustomPlot *customPlot)
   QVector<double> x, y;
   for (int i=1; i<100; ++i)
   {
-    x << 1.0 - 1.0/(double)i;
+    x << 1.0 - 1.0/double(i);
     y << i;
   }
   x << 0.3 << 0.6; // point that should perfectly match grid
@@ -436,7 +442,6 @@ void MainWindow::setupSelectTest(QCustomPlot *customPlot)
   bracket->left->setCoords(-0.2, 0.35);
   bracket->right->setCoords(1.2, 0.65);
   bracket->setLength(22);
-  */
   
   // QCPFinancial:
   QCPFinancial *f = new QCPFinancial(customPlot->xAxis, customPlot->yAxis);
@@ -453,7 +458,12 @@ void MainWindow::setupSelectTest(QCustomPlot *customPlot)
   //customPlot->xAxis->setRangeReversed(true);
   customPlot->xAxis->setRange(-2, 22);
   customPlot->yAxis->setRange(160, 220);
+  */
   
+  // QCPGraph:
+  QCPGraph *g = customPlot->addGraph();
+  g->addData(QVector<double>() << -15 << -12 << -5 << 0 << 0.1 << 0.1 << 5 << 10 << 11,
+             QVector<double>() << 0 << -5 << 5 << 0 << 6 << -2 << 4 << 7 << 5);
   
   connect(customPlot, SIGNAL(beforeReplot()), this, SLOT(selectTestColorMapRefresh()));
 }
@@ -461,7 +471,8 @@ void MainWindow::setupSelectTest(QCustomPlot *customPlot)
 void MainWindow::setupDateTest(QCustomPlot *customPlot)
 {
   customPlot->xAxis->setTicker(QSharedPointer<QCPAxisTicker>(new QCPAxisTickerDateTime));
-  customPlot->xAxis->setRange(QDateTime(QDate(2015, 1, 1)).toTime_t(), QDateTime(QDate(2020, 1, 1)).toTime_t());
+  customPlot->xAxis->setRange(QDateTime(QDate(2015, 1, 1).startOfDay()).toMSecsSinceEpoch()/1000.0,
+                              QDateTime(QDate(2020, 1, 1).startOfDay()).toMSecsSinceEpoch()/1000.0);
   QCPGraph *g = customPlot->addGraph();
   g->addData(QCPAxisTickerDateTime::dateTimeToKey(QDate(2015,1,1)), 1);
   g->addData(QCPAxisTickerDateTime::dateTimeToKey(QDate(2015,6,1)), 2);
@@ -744,7 +755,7 @@ void MainWindow::setupColorMapTest(QCustomPlot *customPlot)
     {
       colorMap->data()->setCell(x, y, qExp(-qSqrt((x-310)*(x-310)+(y-260)*(y-260))/200.0)+
                                       qExp(-qSqrt((x-200)*(x-200)+(y-290)*(y-290))/80.0)-qExp(-qSqrt((x-180)*(x-180)+(y-140)*(y-140))/200.0));
-      colorMap->data()->setAlpha(x, y, qBound(0.0, (1-((x-nx*0.5)*(x-nx*0.5)+(y-ny*0.5)*(y-ny*0.5))/(double)(nx*nx*0.25))*255.0, 255.0));
+      colorMap->data()->setAlpha(x, y, static_cast<unsigned char>(qBound(0.0, (1-((x-nx*0.5)*(x-nx*0.5)+(y-ny*0.5)*(y-ny*0.5))/double(nx*nx*0.25))*255.0, 255.0)));
     }
   }
   
@@ -1074,7 +1085,7 @@ void MainWindow::setupLargeDataSetDelete(QCustomPlot *customPlot)
     QCPGraph *g = customPlot->addGraph();
     QSharedPointer<QCPGraphDataContainer> data(new QCPGraphDataContainer);
     for (int i=0; i<82000; ++i)
-      data->add(QCPGraphData(i, n+rand()/(double)RAND_MAX*0.3));
+      data->add(QCPGraphData(i, n+rand()/double(RAND_MAX)*0.3));
     g->setData(data);
   }
   qDebug() << "create" << timer.nsecsElapsed()/1e6 << "ms";
@@ -1093,7 +1104,7 @@ void MainWindow::setupLargeDataSetDelete(QCustomPlot *customPlot)
     QCPGraph *g = customPlot->addGraph();
     QSharedPointer<QCPGraphDataContainer> data(new QCPGraphDataContainer);
     for (int i=0; i<5000; ++i)
-      data->add(QCPGraphData(i, n+rand()/(double)RAND_MAX*0.3));
+      data->add(QCPGraphData(i, n+rand()/double(RAND_MAX)*0.3));
     g->setData(data);
   }
   customPlot->rescaleAxes();
@@ -1145,7 +1156,7 @@ void MainWindow::setupErrorBarTest(QCustomPlot *customPlot)
 
 void MainWindow::setupDataSelectTest(QCustomPlot *customPlot)
 {
-  qsrand(1);
+  std::srand(1);
   customPlot->setSelectionRectMode(QCP::srmSelect);
   
   QCPGraph *g = customPlot->addGraph();
@@ -1155,21 +1166,21 @@ void MainWindow::setupDataSelectTest(QCustomPlot *customPlot)
   y << 2;
   for (int i=0; i<n/2; ++i)
   {
-    x << i/(double)(n/2-1)*4-5;
-    if (qrand()%(n/25) == 0)
-      y << qrand()/(double)RAND_MAX*7; // generate outliers (must be preserved in adaptive-sampling-algorithm)
+    x << i/double(n/2-1)*4-5;
+    if (std::rand()%(n/25) == 0)
+      y << std::rand()/double(RAND_MAX)*7; // generate outliers (must be preserved in adaptive-sampling-algorithm)
     else
-      y << qCos(qrand()/(double)RAND_MAX*2*M_PI)*qSqrt(-2*qLn(qrand()/(double)RAND_MAX)) + 5*qSin(x[i]);
+      y << qCos(std::rand()/double(RAND_MAX)*2*M_PI)*qSqrt(-2*qLn(std::rand()/double(RAND_MAX))) + 5*qSin(x[i]);
   }
   x << 0.5;
   y << 2;
   for (int i=0; i<n/2; ++i)
   {
-    x << i/(double)(n/2-1)*4+1;
-    if (qrand()%(n/25) == 0)
-      y << qrand()/(double)RAND_MAX*7; // generate outliers (must be preserved in adaptive-sampling-algorithm)
+    x << i/double(n/2-1)*4+1;
+    if (std::rand()%(n/25) == 0)
+      y << std::rand()/double(RAND_MAX)*7; // generate outliers (must be preserved in adaptive-sampling-algorithm)
     else
-      y << qCos(qrand()/(double)RAND_MAX*2*M_PI)*qSqrt(-2*qLn(qrand()/(double)RAND_MAX)) + qSin(5*x[i]);
+      y << qCos(std::rand()/double(RAND_MAX)*2*M_PI)*qSqrt(-2*qLn(std::rand()/double(RAND_MAX))) + qSin(5*x[i]);
   }
   x << 6;
   y << -1;
@@ -1223,7 +1234,7 @@ void MainWindow::setupScatterSkipTest(QCustomPlot *customPlot)
   c->setScatterSkip(4);
   int n = 1000;
   for (int i=0; i<n; ++i)
-    c->addData(qCos(i/(double)n*10*M_PI)*i/(double)n*2+2, qSin(i/(double)n*10*M_PI)*i/(double)n*2);
+    c->addData(qCos(i/double(n)*10*M_PI)*i/double(n)*2+2, qSin(i/double(n)*10*M_PI)*i/double(n)*2);
   
   QCPGraph *g = new QCPGraph(customPlot->xAxis, customPlot->yAxis);
   n = 10000;
@@ -1232,11 +1243,11 @@ void MainWindow::setupScatterSkipTest(QCustomPlot *customPlot)
   y << 2;
   for (int i=0; i<n/2; ++i)
   {
-    x << i/(double)(n/2-1)*4-5;
-    if (qrand()%(n/25) == 0)
-      y << qrand()/(double)RAND_MAX*7; // generate outliers (must be preserved in adaptive-sampling-algorithm)
+    x << i/double(n/2-1)*4-5;
+    if (std::rand()%(n/25) == 0)
+      y << std::rand()/double(RAND_MAX)*7; // generate outliers (must be preserved in adaptive-sampling-algorithm)
     else
-      y << qCos(qrand()/(double)RAND_MAX*2*M_PI)*qSqrt(-2*qLn(qrand()/(double)RAND_MAX)) + 5*qSin(x[i]);
+      y << qCos(std::rand()/double(RAND_MAX)*2*M_PI)*qSqrt(-2*qLn(std::rand()/double(RAND_MAX))) + 5*qSin(x[i]);
   }
   g->setData(x, y);
   g->setScatterStyle(QCPScatterStyle::ssPlus);
@@ -1245,9 +1256,74 @@ void MainWindow::setupScatterSkipTest(QCustomPlot *customPlot)
   customPlot->rescaleAxes();
 }
 
+void MainWindow::setupTimeZoneTest(QCustomPlot *customPlot)
+{
+# if QT_VERSION >= QT_VERSION_CHECK(5, 2, 0)
+  QSharedPointer<QCPAxisTickerDateTime> ticker(new QCPAxisTickerDateTime);
+  customPlot->xAxis->setTicker(ticker);
+  QCPGraph *graph = customPlot->addGraph();
+  
+  double timeStamp = 1577883600; // Wednesday, January 1, 2020 13:00:00 (UTC)
+  graph->setData({timeStamp+3600*0, timeStamp+3600*1, timeStamp+3600*2, timeStamp+3600*3}, {1.0, 2.0, 2.5, 2.75});
+  
+  ticker->setDateTimeFormat("yyyy-MM-dd\nhh:mm:ss");
+  ticker->setTickOrigin(QDateTime(QDate(2020, 1, 15), QTime(17, 17, 17), QTimeZone("UTC")));
+  //qDebug() << QTimeZone::availableTimeZoneIds();
+  ticker->setTimeZone(QTimeZone("Europe/Berlin"));
+  
+  customPlot->xAxis->rescale();
+  customPlot->xAxis->scaleRange(1.1);
+# endif
+}
+
+void MainWindow::setupPolarAxisTest(QCustomPlot *customPlot)
+{
+  presetInteractive(customPlot);
+  customPlot->setPlottingHint(QCP::phCacheLabels, true);
+  customPlot->plotLayout()->clear();
+  QCPPolarAxisAngular *polarAxis = new QCPPolarAxisAngular(customPlot);
+  customPlot->plotLayout()->addElement(0, 0, polarAxis);
+  QSharedPointer<QCPAxisTickerPi> ticker(new QCPAxisTickerPi);
+  ticker->setPiValue(180);
+  ticker->setTickCount(8);
+  polarAxis->setTicker(ticker);
+  polarAxis->setRangeDrag(true);
+  polarAxis->setTickLabelMode(QCPPolarAxisAngular::lmUpright);
+  
+  polarAxis->radialAxis()->setTickLabelMode(QCPPolarAxisRadial::lmUpright);
+  //polarAxis->radialAxis()->setScaleType(QCPPolarAxisRadial::stLogarithmic);
+  //polarAxis->radialAxis()->setTicker(QSharedPointer<QCPAxisTicker>(new QCPAxisTickerLog));
+  //polarAxis->radialAxis()->setRange(0.001, 1e1);
+  polarAxis->radialAxis()->setTickLabelRotation(0);
+  polarAxis->radialAxis()->setAngle(45);
+  
+  polarAxis->grid()->setAngularPen(QPen(QColor(200, 200, 200), 0, Qt::SolidLine));
+  polarAxis->grid()->setSubGridType(QCPPolarGrid::gtAll);
+  //polarAxis->radialAxis()->setRangeReversed(true);
+  
+  QCPPolarGraph *g1 = new QCPPolarGraph(polarAxis, polarAxis->radialAxis());
+  QCPPolarGraph *g2 = new QCPPolarGraph(polarAxis, polarAxis->radialAxis());
+  g2->setPen(QPen(QColor(255, 150, 20)));
+  g2->setBrush(QColor(255, 150, 20, 50));
+  g1->setScatterStyle(QCPScatterStyle::ssStar);
+  for (int i=0; i<360; ++i)
+  {
+    g1->addData(i, qSin(i/360.0*M_PI*8)*8+1);
+    g2->addData(i, qSin(i/360.0*M_PI*6)*2);
+  }
+  polarAxis->radialAxis()->rescale();
+  
+  /*
+  QTimer *timer = new QTimer(this);
+  connect(timer, SIGNAL(timeout()), this, SLOT(tickLabelTestTimerSlot()));
+  timer->start(2000);
+  */
+  //connect(customPlot, SIGNAL(mouseMove(QMouseEvent*)), this, SLOT(mouseMoveRotateAngularTickLabels(QMouseEvent*)));
+}
+
 void MainWindow::setupAdaptiveSamplingTest(QCustomPlot *customPlot)
 {
-  qsrand(1);
+  std::srand(1);
   QCPGraph *g = customPlot->addGraph();
   int n = 200000;
   QVector<double> x, y;
@@ -1255,21 +1331,21 @@ void MainWindow::setupAdaptiveSamplingTest(QCustomPlot *customPlot)
   y << 2;
   for (int i=0; i<n/2; ++i)
   {
-    x << i/(double)(n/2-1)*4-5;
-    if (qrand()%(n/25) == 0)
-      y << qrand()/(double)RAND_MAX*7; // generate outliers (must be preserved in adaptive-sampling-algorithm)
+    x << i/double(n/2-1)*4-5;
+    if (std::rand()%(n/25) == 0)
+      y << std::rand()/double(RAND_MAX)*7; // generate outliers (must be preserved in adaptive-sampling-algorithm)
     else
-      y << qCos(qrand()/(double)RAND_MAX*2*M_PI)*qSqrt(-2*qLn(qrand()/(double)RAND_MAX)) + 5*qSin(x[i]);
+      y << qCos(std::rand()/double(RAND_MAX)*2*M_PI)*qSqrt(-2*qLn(std::rand()/double(RAND_MAX))) + 5*qSin(x[i]);
   }
   x << 0.5;
   y << 2;
   for (int i=0; i<n/2; ++i)
   {
-    x << i/(double)(n/2-1)*4+1;
-    if (qrand()%(n/25) == 0)
-      y << qrand()/(double)RAND_MAX*7; // generate outliers (must be preserved in adaptive-sampling-algorithm)
+    x << i/double(n/2-1)*4+1;
+    if (std::rand()%(n/25) == 0)
+      y << std::rand()/double(RAND_MAX)*7; // generate outliers (must be preserved in adaptive-sampling-algorithm)
     else
-      y << qCos(qrand()/(double)RAND_MAX*2*M_PI)*qSqrt(-2*qLn(qrand()/(double)RAND_MAX)) + qSin(5*x[i]);
+      y << qCos(std::rand()/double(RAND_MAX)*2*M_PI)*qSqrt(-2*qLn(std::rand()/double(RAND_MAX))) + qSin(5*x[i]);
   }
   x << 6;
   y << -1;
@@ -1294,12 +1370,15 @@ void MainWindow::presetInteractive(QCustomPlot *customPlot)
                               QCP::iSelectPlottables|
                               QCP::iSelectOther|
                               QCP::iMultiSelect);
-  customPlot->axisRect()->setRangeDrag(Qt::Horizontal|Qt::Vertical);
-  customPlot->axisRect()->setRangeZoom(Qt::Horizontal|Qt::Vertical);
+  if (customPlot->axisRect())
+  {
+    customPlot->axisRect()->setRangeDrag(Qt::Horizontal|Qt::Vertical);
+    customPlot->axisRect()->setRangeZoom(Qt::Horizontal|Qt::Vertical);
+  }
   connect(customPlot, SIGNAL(mousePress(QMouseEvent*)), this, SLOT(selectionRectChooser(QMouseEvent*)), Qt::UniqueConnection);
 }
 
-void MainWindow::labelItemAnchors(QCPAbstractItem *item, double fontSize, bool circle, bool labelBelow)
+void MainWindow::labelItemAnchors(QCPAbstractItem *item, int fontSize, bool circle, bool labelBelow)
 {
   QList<QCPItemAnchor*> anchors = item->anchors();
   for (int i=0; i<anchors.size(); ++i)
@@ -1366,8 +1445,8 @@ void MainWindow::showSelectTestColorMap(QCustomPlot *customPlot)
           int r = qRed(p[x]);
           int g = qGreen(p[x]);
           int b = qBlue(p[x]);
-          r += 255.0/(dist*0.25+1.0);
-          b += -255.0/(dist*0.25+1.0);
+          r += int(255.0/(dist*0.25+1.0));
+          b += int(-255.0/(dist*0.25+1.0));
           if (qAbs(dist-customPlot->selectionTolerance()) < 0.5)
             g += 255;
           p[x] = qRgb(qBound(0, r, 255), qBound(0, g, 255), qBound(0, b, 255));
@@ -1390,8 +1469,8 @@ void MainWindow::showSelectTestColorMap(QCustomPlot *customPlot)
           int r = qRed(p[x]);
           int g = qGreen(p[x]);
           int b = qBlue(p[x]);
-          r += 255.0/(dist*0.25+1.0);
-          b += -255.0/(dist*0.25+1.0);
+          r += int(255.0/(dist*0.25+1.0));
+          b += int(-255.0/(dist*0.25+1.0));
           if (qAbs(dist-customPlot->selectionTolerance()) < 0.5)
             g += 255;
           p[x] = qRgb(qBound(0, r, 255), qBound(0, g, 255), qBound(0, b, 255));
@@ -1450,6 +1529,16 @@ void MainWindow::mouseMoveRotateTickLabels(QMouseEvent *event)
   mCustomPlot->replot();
 }
 
+void MainWindow::mouseMoveRotateAngularTickLabels(QMouseEvent *event)
+{
+  if (QCPPolarAxisAngular *polarAxis = qobject_cast<QCPPolarAxisAngular*>(mCustomPlot->plotLayout()->element(0, 0)))
+  {
+    //polarAxis->setTickLabelRotation(event->pos().x()-mCustomPlot->width()/2.0);
+    polarAxis->setAngle(std::atan2(event->pos().y()-mCustomPlot->height()/2.0, event->pos().x()-mCustomPlot->width()/2.0)/M_PI*180.0);
+  }
+  mCustomPlot->replot();
+}
+
 void MainWindow::tickLabelTestTimerSlot()
 {
   mCustomPlot->setPlottingHint(QCP::phCacheLabels, !mCustomPlot->plottingHints().testFlag(QCP::phCacheLabels));
@@ -1468,8 +1557,8 @@ void MainWindow::setupMultiAxisRectInteractionsMouseMove(QMouseEvent *event)
 
 void MainWindow::daqPerformanceDataSlot()
 {
-  static qint64 start = QCPAxisTickerDateTime::dateTimeToKey(QDateTime::currentDateTime())*1000;
-  qint64 currentMillisecond = QCPAxisTickerDateTime::dateTimeToKey(QDateTime::currentDateTime())*1000 - start;
+  static qint64 start = qint64(QCPAxisTickerDateTime::dateTimeToKey(QDateTime::currentDateTime())*1000);
+  qint64 currentMillisecond = qint64(QCPAxisTickerDateTime::dateTimeToKey(QDateTime::currentDateTime())*1000) - start;
   static qint64 lastMillisecond = currentMillisecond;
   static int ptsInThisMillisecond = 0;
   if (lastMillisecond != currentMillisecond)
@@ -1482,17 +1571,17 @@ void MainWindow::daqPerformanceDataSlot()
   if (mCustomPlot->graph()->data()->size() > 100000)
     grow = false;
   
-  if (rand()%100000 < 100)
+  if (std::rand()%100000 < 100)
   {
     if (grow)
     {
-      if (rand()%1000 < 280)
+      if (std::rand()%1000 < 280)
         dir = -1;
       else
         dir = 1;
     } else
     {
-      if (rand()%1000 < 450)
+      if (std::rand()%1000 < 450)
         dir = -1;
       else
         dir = 1;
@@ -1510,7 +1599,7 @@ void MainWindow::daqPerformanceDataSlot()
     }
   } else
   {
-    if (rand()%10 < 1)
+    if (std::rand()%10 < 1)
       mCustomPlot->graph(0)->data()->removeBefore(mCustomPlot->graph(0)->data()->constBegin()->key+0.00000001);
     else
       mCustomPlot->graph(0)->data()->remove((mCustomPlot->graph(0)->data()->constEnd()-1)->key);
@@ -1531,12 +1620,12 @@ void MainWindow::daqPerformanceReplotSlot()
   
   int dataPoints = mCustomPlot->graph(0)->data()->size();
   static int lastDataPoints = dataPoints;
-  qint64 now = QCPAxisTickerDateTime::dateTimeToKey(QDateTime::currentDateTime())*1000;
+  qint64 now = qint64(QCPAxisTickerDateTime::dateTimeToKey(QDateTime::currentDateTime())*1000);
   static qint64 lastT = now;
   static QString dataPointFrequency("0 Hz");
   if (now-lastT > 1000)
   {
-    dataPointFrequency = QString::number((dataPoints-lastDataPoints)/(double)(now-lastT)*1000.0)+" Hz";
+    dataPointFrequency = QString::number((dataPoints-lastDataPoints)/double(now-lastT)*1000.0)+" Hz";
     lastT = now;
     lastDataPoints = dataPoints;
   }
@@ -1577,3 +1666,13 @@ void MainWindow::testbedMouseClick(QMouseEvent *event)
 {
   Q_UNUSED(event)
 }
+
+void MainWindow::showReplotTime()
+{
+  static QDateTime lastShow = QDateTime::fromMSecsSinceEpoch(0);
+  if (lastShow.msecsTo(QDateTime::currentDateTime()) > 100) // only update status bar label at most every 100ms
+  {
+    mStatusBarLabel->setText(QString("%1 ms (avg %2 ms)").arg(mCustomPlot->replotTime(false), 0, 'f', 1).arg(mCustomPlot->replotTime(true), 0, 'f', 1));
+    lastShow = QDateTime::currentDateTime();
+  }
+}
